@@ -3,6 +3,7 @@
 //! [`AddressSpace`]: `super::AddressSpace`
 
 pub mod allocator;
+pub mod copy;
 pub mod mapper;
 pub mod protector;
 pub mod reader;
@@ -10,6 +11,7 @@ pub mod remover;
 pub mod writer;
 
 pub use allocator::PteAllocator;
+pub use copy::{CopyFromWalker, CopyToWalker};
 pub use mapper::PteMapper;
 pub use protector::PteProtector;
 pub use reader::PteReader;
